% Code for Generalized HKL presented in the paper "Efficient Rule Ensemble Learning using
% Hierarchical Kernels" by Pratik Jawanpuria, J. Saketha Nath and Ganesh
% Ramakrishnan (IIT Bombay) presented in ICML 2011

% This code is built upon the code of HKL (Francis Bach: Exploring Large Feature
% Spaces with Hierarchical Multiple Kernel Learning. Advances in Neural
% Information Processing Systems (NIPS), 2008) available at
% http://www.di.ens.fr/~fbach/hkl/index.html
% Some of the functions are used as it is from above and some of them are
% discarded/modified.

% In order to do REL using Generalized HKL, you have to just enter the relevant {0,1} feature matrix.
% Please refer to our paper/technical report in order to know the details
% of converting a real and/or nominal valued feature matrix to {0,1}
% feature matrix

% For just running Generalized HKL, above conversion is not required

% This code is licensed under GPL licence and it is free for non-commercial use. 


clear all
clc

%%%%%%PARAMETERS%%%%%%%%
weights = [1 .1 2];
maxactive = 10000;
memory_cache = 4*1e9;
rhonorm = 1.1;
high = 3;
low = -3;
%%%%%%PARAMETERS%%%%%%%%

% ADD LIBSVM PATH
% LOAD DATA FILE containing feature matrix X and label vector Y
% X: n x d matrix where n is number of data points and d is dimension.
% Y is a {-1,1} vector
% For REL, X should be a {0,1} matrix

d = size(X,2);

% default value of conflictMatrix
conflictMatrix = zeros(d,d);

% Concept of Conflict Matrix was added to make the program run faster in
% case we have any prior knowledge about the features of the given dataset.
% It should be changed ONLY when code is run for Rule Ensemble Learning. If
% code is run only for Generalized HKL (i.e. not REL) it SHOULD be a zero matrix.

% Conflict matrix captures the information whether the basic propositions (feature_i==1) &
% (feature_j==1) can coexist in a rule. If yes, the conflictMatrix(i,j) = conflictMatrix(i,j) = 0;
% Else conflictMatrix(i,j) = conflictMatrix(i,j) = 1;
% In case of no prior knowledge, conflictMatrix should be a zero matrix
% which means all basic propositions can occur with each other.
% Example of a case where prior knowledge is present is when features are
% of type A, (NOT A). We know for sure that basic propositions (A==1) and (NOT A ==1)
% cannot coexist in a rule. Hence, this informations helps in traversing
% thru the lattice. Code ran around 20-30% faster when prior knowledge was
% available for conflictMatrix.


range = high - low + 1;
lambdas = logspace(high,low,range);

% Prepare the train-test split - XTrain, YTrain, XTest, YTest

[outputsRho,modelRho,accuraciesRho]  = ghkl_shogun(XTrain,YTrain,lambdas,'hinge','polynomial',weights, 'Xtest',XTest,'Ytest',YTest, ...
                'display',0,'maxactive',maxactive,'memory_cache',memory_cache, 'rhonorm',rhonorm,'conflictMatrix',conflictMatrix);

% In order to see the rules please use the following command:
% graphModel = getGraph(outputsRho{1}.hull(outputsRho{1}.theta > 0,:))
% graphModel will be a r x 1 cell where r is the number of rules
% each row of the cell will contain one rule (indices will represent the
% feature indices of feature matrix). For example: if graphModel = {[1 2 4], [3 4 5]};
% it means the rules are: 
% R1: feature_1 AND feature_2 AND feature_4
% R2: feature_3 AND feature_4 AND feature_5
% Whether these rules favour +ve or -ve class can be see by observing
% corresponding weight vectors (not explicitly calculated here)

% We can do away with the CVX function (cvxLpNormSolver.m) used in
% solve_ghkl_hinge_shogun.m by:
% 1. Writing an equivalent code using LIBSVM (since its 1 node case)
% 2. Initialize hull by more than 1 node in solve_ghkl_shogun (line 50).
% Example: hull = ones(2,data.p); hull(2,1) = 2; % It 2 node initialization.
% Another is: hull = [ones(1,data.p);ones(data.p,data.p)+eya(data.p)]; % It
% root node plus first level node initialization.

